C
C	PROGRAM ACCEL
C
C	TO CALCULATE RADIATIVE ACCELERATION VIA ONE LINE IN EACH
C	RUN IN LTE INCLUDING REDISTRIBUTION EFFECT OR
C	IN NLTE (NO REDISTRIBUTION).
C	IT ADDES THE RESULT TO PREVIOUS RUNS AND ULTIMATELY 
C	ENDS UP WITH THE ACCELERATIONS FOR SEVERAL IONS OF AN ELEMENT 
C	AND SEVERAL ABUNDANCES IN A PARTICULAR MODEL ATMOSPHERE
C
C       INPUT:
C		element.dat  - (7) ATOMIC DATA FOR THE CHEM.ELEMENT
C               model.hub - (3) HUBENY'S ATMOSPHERE MODEL
C		model.hub.5 -(7) HUBENY'S ATMOSPHERE MODEL INPUT FILE
C               fort.ID - (11-80) FILES WITH FLUX AT 'ID' DEPTH POINT
C 		no.line - (9) IDENTIFIER OF THE LINE AND ABUNDANCE
C		              TO BE TREATED
C		line.dat - (8) DATA FOR THE CURRENT LINE
C			IN CASE OF NLTE=1:
C		? - (9) HUBENY'S ATOM (ION) MODEL	
C
C       OUTPUT:
C		accel.out - (2) MORE DETAIL OUTPUT
C		accel.line - (4) DM VERSUS ACCELERATION THROUGH 1 LINE
C		accel.tot - (99) TOTAL ACCELERATION FOR EACH ION AND
C				ABUNDANCE
C
C	COMMENT: CGS UNITS IF NOT SPECIFIED OTHERWISE
C                NDIM - MAIN DIMENSION PARAMETER 
C                NION1 - NUMBER OF IONS DIMENSION
C                NAB1 - NUMBER OF ABUNDANCES DIMENSION
C		 ?? - BE CAUTIOUS
C
C	THINGS TO IMPROOVE: 
C		INCLUDE LID
C		INCLUDE BOUND-FREE TRANSITIONS
C		INCLUDE DELIC AS A SUBROUTINE
C	 	DEFINITION OF ABUNDANCES
C
	PARAMETER (NDIM=90,NION1=9,NAB1=10)
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	CHARACTER*30 INMOD,INMOD5
        CHARACTER*13 A13
	CHARACTER*1 HASH
	DIMENSION ZIP(NION1-1),STA(6,NION1),STAVS(NDIM,NION1)
     P  ,HIPF(2),HEPF(3),STAHI(6,2),STAHEI(6,3),HI(NDIM),HEI(NDIM)
     P	,DM(NDIM),EKONC(NDIM),AKONC(NDIM)
     P	,TEMP(NDIM),VT(NDIM),ILL(NDIM),ACCEL1(NDIM),ACCEL2(NDIM)
     P  ,NLIA(NION1,NAB1),ACCTOT(NDIM,NION1,NAB1),ABUND(NAB1)
     P  ,PRESS(NDIM),ABROSS(NDIM),DENS(NDIM),HLB(NDIM)
     P  ,PLO(NDIM),PUP(NDIM),PION(NDIM)
     P  ,RION(NION1),R1(NION1),RR(NDIM,NION1)
C        	N(I+1)/N(I)=SAHA EQUATION
C             ZP -IN [EV],BOLT.CONST.-IN [ERG/K]
C	SAHA(TE,EK,ZP,S1,S2)=1.2020D9*S2/(S1*(5039.77D0/TE)**2.5D0
C     P	*DEXP(2.302585D0*5039.77D0/TE*ZP)*EK*TE*1.38062D-16)
C------------------------------------------------------------------
        WRITE(6,'(A)')' ACCEL'
        WRITE(6,'(A)')' INPUT: '
C        WRITE(6,'(A)')' flux.20 - OUTPUT OF SYNSPEC WITH FLUX'
        WRITE(6,'(A)')' element.dat  - ATOMIC DATA FOR THE CHEM.ELEMENT'
        WRITE(6,'(A)')' model.hub - HUBENY ATMOSPHERE MODEL'
        WRITE(6,'(A)')'	model.hub.5 - HUBENY MODEL INPUT FILE'
        WRITE(6,'(A)')' line.dat - DATA FOR THE CURRENT LINE'
        WRITE(6,'(A)')' no.line - IDENTIFIER OF THE LINE AND ABUNDANCE'
        WRITE(6,'(A)')' fort.ID - FILES WITH FLUX AT ID DEPTH POINT'
        WRITE(6,'(A)')'			IN CASE OF NLTE=1:'
        WRITE(6,'(A)')'	? -  HUBENY ATOM (ION) MODEL'	
        WRITE(6,'(A)')' OUTPUT: '
        WRITE(6,'(A)')' accel.out - MORE DETAIL OUTPUT'
        WRITE(6,'(A)')' accel.line - DM VERSUS ACCELERATION'
        WRITE(6,'(A,A/)')' accel.tot - TOTAL ACCELERATION FOR EACH ION'
     P  ,' AND ABUNDANCE'
	OPEN(2,FILE='accel.out',STATUS='UNKNOWN')
	OPEN(4,FILE='accel.line',STATUS='UNKNOWN')
C	CONSTANTS AND UNITS FROM NIST(2002)
	BOL=1.3806503D-16
	CLIGHT=2.99792458D10
	HJED=1.66053873D-24
	PLANK=6.62606876D-27
        PI=3.14159265358D0
C	1EV=X ERG, X CM^-1; 1ERG= X CM^-1 
	EVERG=1.602176462D-12
	EVCM=8.06554477D3
	ERGCM=5.03411762D15
C-----------------------------------------------------------------
C	IMODEL=1 (HUBENY'S MODEL)
C	IMODEL=0 (KURUCZ'S MODEL)
	IMODEL=1
	IF (IMODEL.EQ.1) THEN
C
C	      READ HUBENY'S MODEL OF THE ATMOSPHERE
C
C	  DM - MASS DEPTH COORDINATE
C	  TEMP - TEMPERATURE
C	  EKONC - ELECTRON NUMBER DENSITY
C	  AKONC - (ALL) ATOM NUMBER DENSITY
C	  DENS - DENSITY
C
	  INMOD='model.hub'
	  INMOD5='model.hub.5'
          CALL HMODEL(INMOD,NDIM,NBOD
     P    ,DM,TEMP,EKONC,AKONC,DENS)
	  TEFF=0.D0
	  GLOG=0.D0
	  OPEN(7,FILE=INMOD5,STATUS='OLD')
	  READ(7,*)TEFF,GLOG
	  CLOSE(7)
	ENDIF
        IF (IMODEL.EQ.0) THEN
C	      KURUCZ MODEL OF THE ATMOSFERE
	  INMOD='model'
          CALL AMODEL(INMOD,NDIM,NBOD,TEFF,GLOG
     P    ,DM,TEMP,PRESS,EKONC,ABROSS,AKONC,DENS,HLB)
        ENDIF
C-----------------------------------------------------------------------
C
C	NBOD - NO. OF MODEL DEPTH POINTS
C	IZMAZ - DUMMY
C	INTERP - DUMMY
C	NABUND - NUMBER OF ABUNDANCES
C	ABUND - ABUNDANCES, N(EL)/N(H), FOR WHICH THE ACCELERATIONS ARE 
C		CALCULATED
C	NION - NUMBER OF IONS FOR WHICH THE ACCELERATIONS ARE CALCULATED 
C	NLTE - =1 IF THE NLTE POPULATIONS ARE TO BE CONSIDERED
C		FROM TLUSTY MODEL INSTEAD OF LTE (NLTE=0)
C	IDLTE - DEPTH POINT BELOW WHICH POPULATIONS ARE TAKEN IN LTE
C	        INSPITE OF NLTE=1 OPTION (ONLY IF NLTE=1)
C       IRED - =0 NO REDISTRIBUTION
C	IRED - =1 OUR WAY OF REDISTRIBUTION, 
C	        1/[RPAC*DEXP(-DELTAE/RPARB/KT-DELTAE/RPARA) 
C		PORTION OF THE MOMENTUM IS REDISTRIBUTED TO 
C		THE NEXT HIGHER ION
C       IRED - =2 REDISTRIBUTION AFTER
C               GONZALES AT AL.1995, LINES WITH EUP>(ZIP-RPARA) ARE
C               ASCRIBED TO THE NEXT HIGHER IONIZATION STATE
C		(RPARB,RPARC-DUMMY)
C	RPARA - REDISTRIBUTION PARAMETER, ENERGY IN CM^-1
C       RPARB - REDISTRIBUTION PARAMETER, MULTIPLE OF KT
C       RPARC - REDISTRIBUTION PARAMETER OF THE ORDER OF 4.E1
C       ABHYD - H ABUNDANCE, ABHEL - He ABUNDANCE, N/N(TOT)
C       IVTURB - OPTION FOR DIF. KOEF. OF A TURBULENCE:
C                                       0-HOMOGENEOUS
C                                       1-OPTIONAL IN EACH DEPTH POINT
C                                       2-AS A 1/(GASS NUMBER DENSITY)
C	VT - VELOCITY OF THE TURBULENCE [CGS]
C 	IAT - ELEMENT ATOM NUMBER, HMC - MASS NUMBER
C	ZIP - FIRTS 8 IONIZATION POTENTIALS [EV]
C	IPART - OPTION OF PARTITION FUNCTION 
C		[1-BUIT IN DWORETSKY&SMALLEY, 2-IRWIN]
C	STA - PARTITION FUNCTION COEFFICIENTS FOR 8+1 STATES
C		AFTER IRWIN, 1981, ApJS 45,621	
	OPEN(7,FILE='element.dat',STATUS='OLD')
	READ(7,'(/)')
        READ(7,*)IZMAZ,INTERP,NABUND,NION,NLTE,IDLTE,IRED
     P  ,RPARA,RPARB,RPARC
        READ(7,*)
        INTERP=0
	IF(NLTE.NE.0.AND.IRED.NE.0)THEN
          WRITE(6,*)' ERROR, REDISTRIBUTION AND NLTE WERE SIMULTANEOUS'
	  STOP
	ENDIF
        READ(7,*)(ABUND(I),I=1,NABUND)
        READ(7,*)
        READ(7,*)ABHYD,ABHEL
	READ(7,*)	
	READ(7,*)IVTURB
C	MUST BE THE SAME AS IN THE SYNTHETIC SPECTRA! SO: 
	IVTURB=0
        READ (7,*)
        IF(IVTURB.EQ.0)THEN
          READ(7,*)VT00
          DO 10 I=1,NBOD
            VT(I)=VT00
10        CONTINUE
        ENDIF
        IF(IVTURB.EQ.1)THEN
          READ(7,*)(VT(I),I=1,NBOD)
        ENDIF
        IF(IVTURB.EQ.2)THEN
          READ(7,*)VT(1)
        ENDIF
        READ(7,*)
	READ(7,*)IAT,HMC
	READ(7,*)
        READ (7,*)(ZIP(I),I=1,8)
	READ(7,*)
        READ(7,*)IPART
        READ(7,*)
        DO 20 J=1,NION1
	 READ (7,*)(STA(I,J),I=1,6)
20      CONTINUE
	CLOSE(7)
        IF(DABS(ABHEL-0.089D0).GT.0.02D0.OR.
     P    DABS(ABHYD-0.911D0).GT.0.02D0) THEN
          WRITE(6,*)' ABNORMAL H OR HE ABUNDANCE!
     P    YOU HAVE TO CHECK IF THIS IS IN AGREEMENT WITH
     P    THE MODEL USED AS THESE ABUNDANCES AFFECT THE MODEL AND E.G.
     P    THE CALCULATION OF NUMBER
     P    DESITY FROM DENSITY IN THE CODE. 
     P    NORMAL H AND HE ABUNDANCES ARE 0.9 AND 0.1, RESPECTIVELY'
          PAUSE
        ENDIF
        IF(IVTURB.EQ.2)THEN
          DO 30 I=1,NBOD
            VT(I)=VT(1)*AKONC(1)/AKONC(I)
30        CONTINUE
	ENDIF
C-----------------------------------------------------------------------
C	      PARTITION FUNCTIONS OF THE ELEMENT AT DIFFERENT DEPTHS
C	      STAVS(I,J) :  I-DEPTH POINT, J- ION
	IF(IPART.EQ.1)THEN
	  CALL PFDWOR(NDIM,NBOD,NION1,NION,IAT,TEMP,EKONC,STAVS)
        ELSE
          CALL PFIRW(NDIM,NBOD,NION1,NION,STA,TEMP,STAVS)
        ENDIF
C		POLYNOMIAL PARTITION FUNCTION COEFFICIENTS FOR 
C		HI,HII   A    HEI,HEII,HEIII 
        DATA STAHI/-2.61655891D2,1.63428326D2,-4.06133526D1,5.03282928D0
     P	 ,-3.10998364D-1,7.66654594D-3,0.0D0,0.0D0,0.0D0,0.0D0,0.0D0
     P	 ,0.0D0/
        DATA STAHEI/-3.76575219D-1,2.33951687D-1,-5.79755525D-2
     P	 ,7.16333160D-3,-4.41302573D-4,1.08442997D-5,6.93147179D-1
     P	 ,9.29636701D-10,-2.30049742D-10,2.83829746D-11,-1.74590774D-12
     P	 ,4.28355287D-14,0.0D0,0.0D0,0.0D0,0.0D0,0.0D0,0.0D0/
        WRITE(2,*)'   TEMP      HI        HEI    NUMBER DENSITIES'
	DO 227 I=1,NBOD
	  SALA=DLOG(TEMP(I))
C		HIPF - H PARTITION FUNCTION
	  DO 210 J=1,2
	    HIPF(J)=DEXP(STAHI(1,J)+STAHI(2,J)*SALA
     P      +STAHI(3,J)*SALA**2.D0+STAHI(4,J)*SALA**3.D0
     P      +STAHI(5,J)*SALA**4.D0+STAHI(6,J)*SALA**5.D0)
210	  CONTINUE
C               HEPF - HE PARTITION FUNCTION
	  DO 220 J=1,3
	    HEPF(J)=DEXP(STAHEI(1,J)+STAHEI(2,J)*SALA+STAHEI(3,J)
     P	    *SALA**2.D0+STAHEI(4,J)*SALA**3.D0+STAHEI(5,J)*SALA**4.D0
     P	    +STAHEI(6,J)*SALA**5.D0)
220	  CONTINUE
C               HI - HI NUMBER DENSITY
C               HEI - HeI NUMBER DENSITY
	  HI(I)=1.D0/(1.D0+SAH(TEMP(I),EKONC(I),13.598D0,HIPF(1)
     P	  ,HIPF(2)))*AKONC(I)*ABHYD
	  HEI(I)=1.D0/(1.D0+SAH(TEMP(I),EKONC(I),24.587D0,HEPF(1)
     P	   ,HEPF(2))*(1.D0+SAH(TEMP(I),EKONC(I),54.416D0,HEPF(2)
     P	   ,HEPF(3))))*AKONC(I)*ABHEL
	  WRITE(2,225)TEMP(I),HI(I),HEI(I)
225       FORMAT(F8.0,2E10.3)
227	CONTINUE
C-----------------------------------------------------------------------
C       RR(I,J)- POPULATION OF THE J-TH ION/TOTAL ELEMENT POLULATION
C		 AT I-TH DEPTH
C	R1(J)= N(J)/N(1)        DUMMY FIELD HERE
C	RION(J)= N(J)/N(J-1)    DUMMY FIELD HERE
C-----------------------------------------------------------------------
	CALL ZASTUP(DM,TEMP,EKONC,ZIP,STAVS,RR,RION,R1,NDIM,NBOD
     P  ,NION1,NION)
C----------------------------------------------------------------- 
C 	LINE DATA LIKE IN THE SYNSPEC CODE 
C 	DLL0-LAMBDA IN [A] 
C	FREQ0 -FREQUENCY [1/S]
C 	STIO-DEGREE OF ION [1.-NEUTRAL,...] 
C 	BIJ-TRANSITION PROBABILITY [1/SEK]=EINST.COEF (WE INCLUDE 
C 	MULTIPLYING BY THE NUMBER OF GATHERED LINES IN IT) 
C 	QLO -QVANTUM NUMBER -J OF THE LOWER LEWEL[=>STAT.WEIGHT=2*J+1] 
C 	QUP -QVANTUM NUMBER -J OF THE UPPER LEWEL ... 
C	VAHA-NUMBER OF LINES GATHERED INTO THIS ONE LINE
C	IPCL -ORDER (IDENTIFIER) OF THE CURRENT LINE WHICH IS TREATED 
C 		FOR RADIATIVE ACCLERATION IN THE LINELIST
C	IABUND -ORDER (IDENTIFIER) OF THE CURRENT ABUNDANCE
C	NLIN   -NUMBER OF ALL LINES
C-------------------------------
        OPEN(9,FILE='no.line',STATUS='OLD')
        READ(9,*)
        READ(9,*,ERR=110)IPCL1,IABUND1,NLIN
        CLOSE (9)
	IF(IPCL1.EQ.1)THEN
	  IABUND=IABUND1-1
	  IPCL=NLIN
	ELSE
 	  IABUND=IABUND1
	  IPCL=IPCL1-1
	ENDIF
	OPEN(8,FILE='line.dat',STATUS='OLD')
	VAHA=1.D0
	READ(8,*,ERR=110,END=110)DLL,COD,GF,ELO,QLO,EUP,QUP
     P	,GR0,GS0,GW0,INEXT
	DLL0=DLL*10.D0
	FREQ0=CLIGHT/DLL*1.D7
	GLO=2.D0*QLO+1.D0
	GUP=2.D0*QUP+1.D0
	STIO=(COD-DINT(COD))*100.D0+1.D0
	IION=INT(STIO+0.5D0)
C	  WRITE(6,*)'DINT NINT INT',DINT(COD),NINT(COD),INT(COD)
C	  BACHA DINT JE REAL*8, INT A NINT SU INTEGERY
C	  WRITE(*,*)'ZMAZ,GR,GS,GW=1,A7',ZMAZ,GR0,GS0,GW0,A7
C	  WRITE(6,*)'COD STIO',COD,STIO
	BIJ=VAHA*DEXP(2.302585D0*GF)/GLO*DLL0/7.484D-7
	CLOSE(8)
	GOTO 120
110	WRITE(6,*)' ERROR OR END IN THE LINE-LIST'
	STOP
120	CONTINUE
C	DATA IN/'11','12','13','14','15','16','17','18','19','20'
C     P  ,'21','22','23','24','25','26','27','28','29','30'
C     P  ,'31','32','33','34','35','36','37','38','39','40'
C     P  ,'41','42','43','44','45','46','47','48','49','50'
C     P  ,'51','52','53','54','55','56','57','58','59','60'
C     P  ,'61','62','63','64','65','66','67','68','69','70'
C     P  ,'71','72','73','74'/
C-----------------------------------------------------------------------
C	IN CASE OF NLTE EXTRACT THE LOWER AND UPPER LEVEL POPULATIONS
C	AS WELL AS TOTAL ION POPULATION FROM TLUSTY OUTPUT
C	IION=1,2,... FOR NEUTRAL, ONCE IONIZED,... 
C	PLO, PUP, PION -POPULATIONS OF THE LOWER, UPPER LEVEL AND ION
C	NLTE2=0 IF NO NLTE POPULATION IS AVAILABLE IN TLUSTY
	NLTE2=0
	IF (NLTE.EQ.1)THEN
	  CALL PNLTE(IAT,IION,ELO,GLO,EUP,GUP,PLO,PUP,PION,NLTE2
     P    ,STAVS,INMOD,INMOD5,NDIM,NBOD,NION1,ZIP)
	ENDIF
C-----------------------------------------------------------------------
C	TAKE INTO ACCOUNT THAT FORT.LD CORRESPOND TO LD-10 SYNSPEC
C	DEPTH POINT WHICH CORRESPOND TO LD-10 TLUSTY MODEL DEPTH POINT
C	OR LD-10+1 KURUZC'S MODEL DEPTH POINT
C	(IF HIS MODEL WAS USED IN SYNSPEC. HUBENY OMITS FIRST KURUCZ 
C	POINT) 
C	HERE WE WORK IN ORIGINAL MODEL DEPTH POINTS ,ID1, EITHER 
C	KURUCZ'S OR HUBENY'S ONES (ID = SYNSPEC DEPTH POINTS) 	
        WRITE(4,125)TEFF,GLOG,IAT
        WRITE(4,130)1
        WRITE(4,140)ABUND(IABUND)
        WRITE(4,150)IION
        WRITE(4,160)
125     FORMAT('# TEFF, LOGG, ELEMENT :',F10.0,F10.2,I10)
130     FORMAT('# NO. LINES :',I10)
140     FORMAT('# ABUNDANCE :',E10.3)
150     FORMAT('# ION       :',I10)
160 	FORMAT('# ID   DM     RAD.ACC.I  RAD.ACC.I+1')
        IF (IMODEL.EQ.1) THEN
	  IUPPER=NBOD-1
	  ILOWER=2
	ELSE
          IUPPER=NBOD-1
          ILOWER=3
	ENDIF	
        WRITE(2,'(A,E13.6)')' LAMBDA[A]= ',DLL0
C
C		BEGINNING OF THE DEPTH CYCLE----------------------------
	DO 600 ID1=IUPPER,ILOWER,-1
          IF (IMODEL.EQ.1) THEN
	    ID=ID1
          ELSE
            ID=ID1-1
          ENDIF   
	  ID10=ID+10
	  WRITE(2,*)
	  WRITE(2,*)' THE FILE : ',ID10
	  WRITE(2,200)ID1,DM(ID1)
200	  FORMAT(I3,' DEPTH POINT.,   DM=',E10.3)
C-----------------------------------------------------------------------
C	CALCULATION OF INTEGRALS (IS IN ANGSTROMS AS SCALES DIVIDE OUT):
C	/                          /
C	| FLUX*VOIGHT*DELTA(FREQ)/ | VOIGHT*DELTA(FREQ)
C	/                          /
C		SINTEG -INTEGRAL THROUGH 1 LINE
C		SUMS -AREA OF VOIGHT PROFILE
C		OMOC -CONST=0.,0.5
C		ACCEL -ACCELERATION THROUGH 1 LINE
C		WAVEL -LABMDA IN ANGSTREM
C		FLUX = 4*PI*EDD.FLUX IN [ERG/S/CM**2/(1/S)]
C
230	  READ(ID10,240,END=300)HASH,WAVEL,FLUX
240	  FORMAT(A1,F12.4,E12.4)
C	  WRITE(6,*)' WAVEL FLUX',WAVEL,FLUX
	  IF (HASH.EQ.'#')THEN
C	    GR=GR0
C	    GS=GS0
C	    GW=GW0
	    PCP=IAT
	    CALL UTLM(DLL0,EUP,PCP,HMC,STIO
     P	    ,ZIP(IION),HI(ID1),HEI(ID1),EKONC(ID1),TEMP(ID1)
     P      ,VT(ID1),GR,GR0,GS,GS0,GW,GW0,AA,DOP)
	    SUMS=0.D0
	    OMOC=0.D0
	    SINTEG=0.D0
	    POMOC1=0.D0
	    WAVEL1=0.D0
	    FLUX1=0.D0
	    GOTO 230
	  ENDIF
	  POMOC=VOIGT(WAVEL,DLL0,AA,DOP)
	  SINTEG=OMOC*(POMOC*FLUX+POMOC1*FLUX1)*(WAVEL-WAVEL1)+SINTEG
	  SUMS=OMOC*(POMOC+POMOC1)*(WAVEL-WAVEL1)+SUMS
C	  WRITE(6,*)'SINTEG,SUMS',SINTEG,SUMS
C	  WRITE(6,*)' LAMDA VOIGT',WAVEL,POMOC
	  OMOC=0.5D0
	  WAVEL1=WAVEL
	  FLUX1=FLUX
	  POMOC1=POMOC
	  GOTO 230
C-----------------------------------------------------------------------
C	  CALCULATION OF BOUND 
C		BOUND - FREQUENCY INDEPENDENT TERM OF RAD. ACCELERATION
300	  IF(NLTE2.EQ.0.OR.ID1.GE.IDLTE)THEN
	    POPUL=BOLT(TEMP(ID1),STAVS(ID1,IION),GLO,ELO,1)
	    BOUND=(1.D0-DEXP(-1.43871D0/TEMP(ID1)/DLL0/1.D-8))
     P	    *BIJ*POPUL*PLANK/HMC/HJED/DLL0/1.D-8
	  ELSE
	    BOUND=PLANK*FREQ0/HMC/HJED/CLIGHT
     P      *(PLO(ID1)-PUP(ID1)*GLO/GUP)/PION(ID1)*BIJ
            WRITE(2,440)PLO(ID1),PUP(ID1),PION(ID1)
	  ENDIF
C	  	OLD VERSION WITH UNKNOWN CORRECTION FOR 
C		\INT(VOIGHT F.)dFREQ .LT. SQRT(PI)*DOP.W.[1/S] 
C	  ACCEL=SINTEG*BOUND/SUMS
C		NEW VERSION UNDERESTIMATING RA IN SUCH CASE
C	  	ASSUMING FIXED UNITS!! AND NORMALIZING UNITY TO THEM
	  UNNORM=SQRT(PI)*DOP/1.D8/CLIGHT*DLL0*DLL0
	  ACCEL=SINTEG*BOUND/UNNORM
	  IF(ZIP(IION).LT.EUP/EVCM)THEN
	    WRITE(2,460)GR,GS,GW,AA,DOP
	  ELSE
	    WRITE(2,470)GR,GS,GW,AA,DOP
	  ENDIF
	  UNITY=SUMS/SQRT(PI)/DOP*1.D8*CLIGHT/DLL0/DLL0
	  WRITE(2,450)BOUND,SINTEG,UNITY,ACCEL
C	NOTICE THAT IF THE SPECTRUM IS SUFFICIENTLY WIDE UNITY=1
C	\INT(VOIGHT F.)dFREQ HAS NORMALIZATION= SQRT(PI)*DOP.W.[1/S]
440       FORMAT(' PLO=',E10.3,' PUP=',E10.3,' PION=',E10.3) 
450       FORMAT(' BOUND=',E10.3,' SINTEG=',E10.3,' UNITY=',E10.3,
     P    ' ACCEL=',E10.3)
460       FORMAT(' GR=',E8.2,' GS=',E8.2,' GW=',E8.2,
     P    ' DAMP.=',E8.2,' DOP.HW.',E8.2,' AUTO')
470       FORMAT(' GR=',E8.2,' GS=',E8.2,' GW=',E8.2,
     P    ' DAMP.=',E8.2,' DOP.HW.=',E8.2)
C		REDISTRIBUTION EFFECT-------------------
          IF(IRED.GT.0)THEN
  	    IF(IRED.EQ.1)THEN
C	      OUR REDISTRIBUTION
	      DELTAE=(ZIP(IION)*EVCM-EUP)
	      UU=DELTAE*(1.D0/RPARA+1.D0/(RPARB*BOL*TEMP(ID1)*ERGCM))
	      RUI=1.D0/(RPARC*DEXP(-UU)+1.D0)
	      ACCELA=ACCEL*RUI
              IIONR=IION+1
              IF(IIONR.GT.NION)THEN
	        ACCELB=0.D0	
              ELSE	
	        ACCELB=ACCEL*(1.D0-RUI)*RR(ID1,IION)/RR(ID1,IION+1)
              ENDIF
            ENDIF
            IF(IRED.EQ.2)THEN
C	      GONZALES ET AL. REDISTRIBUTION
	      ENCOL=ZIP(IION)*EVCM-RPARA
              IIONR=IION+1
              IF(EUP.GT.ENCOL)THEN	
                ACCELA=0.D0
                IF(IIONR.GT.NION)THEN
                  ACCELB=0.D0
                ELSE
                  ACCELB=ACCEL*RR(ID1,IION)/RR(ID1,IION+1)
                ENDIF
              ELSE
                ACCELA=ACCEL         
                ACCELB=0.D0
              ENDIF
            ENDIF
          ELSE
C		NOREDISTRIBUTION
            ACCELA=ACCEL
            ACCELB=0.D0
	  ENDIF
	  WRITE(4,550)ID1,DM(ID1),ACCELA,ACCELB
550	  FORMAT(I3,3E10.3)
600	CONTINUE
C               END OF THE DEPTH CYCLE----------------------------------
	CLOSE(2)
	CLOSE(4)
C-----------------------------------------------------------------------
C	NLIA -NUMBER OF LINES OF A PARTICULAR ION AND ABUNDANCE 
C		TREATED SO FAR
        OPEN(99,FILE='accel.tot',STATUS='UNKNOWN')
        OPEN(4,FILE='accel.line',STATUS='OLD')
C		ID=DEPTH,II=ION,IA-ABUNDANCE
        READ(99,*)
        READ(4,*)
	READ(99,610)A13,((NLIA(II,IA),II=1,NION),IA=1,NABUND)
610 	FORMAT(A13,200I10)
	READ(99,'(//)')
	READ(4,'(///)')
	ID=1
620     READ(99,*,END=630)ILL(ID),DM(ID),
     P  ((ACCTOT(ID,II,IA),II=1,NION),IA=1,NABUND)
	READ(4,*,END=630)ILL(ID),DM(ID),ACCEL1(ID),ACCEL2(ID)
	ID=ID+1
	GOTO 620
630	N=ID-1
        CLOSE(4)
	CLOSE(99)
	NLIA(IION,IABUND)=NLIA(IION,IABUND)+1
        OPEN(99,FILE='accel.tot',STATUS='UNKNOWN')
        WRITE(99,125)TEFF,GLOG,IAT
	WRITE(99,640)((NLIA(II,IA),II=1,NION),IA=1,NABUND)
	WRITE(99,645)((ABUND(IA),II=1,NION),IA=1,NABUND)
	WRITE(99,650)((II,II=1,NION),IA=1,NABUND)
        WRITE(99,635)
635     FORMAT('# ID   DM     RAD.ACC. FOR GIVEN MODEL, ION, ABUNDANCE'
     P  ,' AND NUMBER OF LINES')
640	FORMAT('# NO. LINES :',200I10)
645	FORMAT('# ABUNDANCE :',200E10.3)
650     FORMAT('# ION       :',200I10)
	DO 670 ID=1,N
	  DO 660 II=1,NION
            DO 655 IA=1,NABUND
C              IF (NLIA(II,IA).EQ.0.AND.IPCL.EQ.1.AND.IABUND.EQ.1)
C     P        ACCTOT(ID,II,IA)=0.D0
	      IF (II.EQ.IION.AND.IA.EQ.IABUND) THEN
C                IF (NLIA(II,IA).EQ.1)ACCTOT(ID,II,IA)=0.D0
                ACCTOT(ID,II,IA)=ACCTOT(ID,II,IA)+ACCEL1(ID)	  
		IF((IION+1).LE.NION)THEN
		  ACCTOT(ID,II+1,IA)=ACCTOT(ID,II+1,IA)+ACCEL2(ID)
                ENDIF
	      ENDIF
655         CONTINUE
660       CONTINUE
          WRITE(99,680)ILL(ID),DM(ID),
     P    ((ACCTOT(ID,IIK,IAK),IIK=1,NION),IAK=1,NABUND) 
670	CONTINUE
        CLOSE(99)
680	FORMAT(I3,200E10.3)
	STOP
	END
C-----------------------------------------------------------------------
	DOUBLE PRECISION FUNCTION VOIGT(DL,DL0,A,DOP)
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C	 REAL*8 VOIGT0
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C								       C
C	     DL  - CHOOSEN LAMBDA [ANGSTROM]         		       C
C	     DL0 - LINE CENTER [ANGSTROM]			       C
C	     A	 - DAMPING PARAMETER				       C
C	     DOP - DOPLER. HALFWIDTH [1/S]			       C
C								       C
C	     OUTPUT - VOIGT PROFIL				       C
C								       C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
	C=2.997925D10
	X=1.D8*(C/DL-C/DL0)/DOP
	POM=A
	VYST=VOIGT0(X,POM)
	VOIGT=VYST
	RETURN
	END
C-----------------------------------------------------------------------
	DOUBLE PRECISION FUNCTION VOIGT0(X,Y)
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C      IMPLICIT REAL*8 (A-H,O-Z)
C      REAL*8 VOIGT0
C      REAL*4 X,Y
C   HUMLICEK VOIGT FUNCTION ALGORITHM  (LOWER PRECISION VERSION)
      DIMENSION T(6),C(6),S(6)
		DATA T/.314240376,.947788391,1.59768264,2.27950708,3.020
     /63703,3.8897249/, 	  C/1.01172805,-.75197147,1.2557727E-2,
     /1.00220082E-2,-2.42068135E-4,5.00848061E-7/,	     S/1.393237,
     /.231152406,-.155351466,6.21836624E-3,9.19082986E-5,-6.27525958E-7/
      WR=0.
      Y1=Y+1.5
      Y2=Y1*Y1
      DO 3 I=1,6
      R=X-T(I)
      D=1./(R*R+Y2)
      D1=Y1*D
      D2=R*D
      R=X+T(I)
      D=1./(R*R+Y2)
      D3=Y1*D
      D4=R*D
      WR=WR+C(I)*(D1+D3)-S(I)*(D2-D4)
    3 CONTINUE
      VOIGT0=DBLE(WR)
      RETURN
      END
C-----------------------------------------------------------------------
	DOUBLE PRECISION FUNCTION BOLT(TEMP,G1,G2,POT,IRIAD)
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C	      TEMP -TEMPERATURE [KELVIN]			       C
C	      G1   -STATISTICAL WEIGHT OF THE LOWER LEVEL-m	       C
C	      G2   -STATISTICAL WEIGHT OF THE UPPER LEVEL-n	       C
C	      POT  -ENERGY DIFFERENCE (UPPER-LOWER LEVEL) [EV], POT>0  C
C	      OUTPUT -POPULATION RATIO N(n)/N(m) 		       C
C---------------------OR IF:            -------------------------------C
C	      TEMP -TEMPERATURE [KELVIN]			       C
C	      G2   -STATISTICAL WEIGHT OF THE LEVEL-n		       C
C	      G1   -PARTITION FUNCTION OF THE ION                      C
C	      POT  -EXCITATION POTENCIAL OF THE n-TH LEVEL [EV]	       C
C		   (WITH RESPECT TO GROUND STATE), POT>0	       C
C	      OUTPUT -RATIO N(n)/N, (n-TH LEVEL POPULATION TO	       C
C		    TO THE ION POPULATION			       C
C-----------------------OR IF------------------------------------------C
C	      IRIAD=1  POT IS IN [1/CM]         		       C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
	EVCM=8.06554477D3
	IF(IRIAD.EQ.1)THEN
	  BOLT=G2/(G1*DEXP(2.302585D0*5039.77D0/TEMP*POT/EVCM))
	ELSE
	  BOLT=G2/(G1*DEXP(2.302585D0*5039.77D0/TEMP*POT))
	ENDIF
	RETURN
	END
C-----------------------------------------------------------------------
	SUBROUTINE UTLM(DL0,EP,S,AM,Z,CHI,HI,HEI,EKONC,TEMP,VT
     P  ,GR,GR0,GS,GS0,GW,GW0,A,DOP)
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C	     DL0   -LINE CENTER [ANGSTROM]			       C
C	     EP    -EXCIT.POTENTIAL OF THE UPPER LEVEL [1/CM]	       C
C	     S	   -ATOM NUMBER 				       C
C	     AM    -MASS NUMBER 				       C
C	     Z	   -DEGREE OF IONIZATION, Z=1. FOR NEUTRAL...	       C
C	     CHI   -IONIZ. POTENTIAL OF THE ION [EV] 		       C
C	     HI    -NEUTR. HYDROGEN NUMBER DENSITY [CGS]	       C
C	     HEI   -NEUTR. HELIUM NUMBER DENSITY [CGS]		       C
C	     EKONC -ELECTRON NUMBER DENSITY [CGS]		       C
C	     TEMP   TEMPERATURE [KELVIN]			       C
C	     VT    -TUBULENCE VELOCITY [CGS]    		       C
C	     GR    -RADIATIVE DAMPING				       C
C	     GS    -STARK DAMPING				       C
C	     GW    -VAN DER WAALSOV DAMPING			       C
C								       C
C	     OUTPUT:                                                   C
C		A   - DAMPING PARAMETER                                C
C		DOP - DOPLER. HALF-WIDTH [1/S]                         C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
	EVCM=8.06554477D3
	C=2.997925D10
	DLL0=1.E-8*DL0
	BOL=1.380622D-16
C	     ZIARIVY UTLM
	IF(GR0.GT.0.D0)THEN
	  GR=DEXP(2.302585D0*GR0)
	ELSE
	  GR=2.4734D-22*C*C/DLL0/DLL0
C	     V PRIPADE AK POZNAS SIRKU AUTOIONIZACNEJ HLADINY GRA
C	     V [EV] => GR=1.52E12*GRA
	ENDIF
	EFF2=Z*Z*13.595D0/(CHI-EP/EVCM)
	IF(EFF2.LT.0.D0.OR.EFF2.GT.25.D0)EFF2=25.D0
C	     STARKOVSKY UTLM
C	 IF(CHI.LE.EP/EVCM) THEN
C	     AUTOIONIZ. HORNA HLADINA ? =>BERIEME JU AKO EFF2=25
C            AJ VO VAN DER WAALSOVOM PRE S<20 (AT.CISLO)
C	   EFF2=0.D0
C	 ENDIF
	IF(GS0.NE.0.D0)THEN
	  GS=DEXP(2.302585D0*GS0)
	ELSE
	  GS=1.0D-8*EFF2**2.5D0
	ENDIF
C	     VAN DER WAALSOV UTLM
	IF (INT(S).LT.21.OR.INT(S).GT.28)THEN
          R2=2.5D0*EFF2*EFF2/Z/Z
	ELSE
	  R2=(45.D0-S)/Z
	ENDIF
	IF(GW0.NE.0.D0)THEN
	  GW=DEXP(2.302585D0*GW0)
	ELSE
	  GW=4.5D-9*R2**0.4D0
	ENDIF
C	     DOPL.POLOSIRKA VO FREKVENCIACH
	DOP=DSQRT(2.D0*BOL*TEMP/(AM*1.672614D-24)+VT*VT)/DLL0
C	     FRAME DAMPING PARAMETER
	A=(GR+GS*EKONC+GW*(HI+0.42D0*HEI)*(TEMP/1.D4)**0.3D0)/
     P  (4.D0*3.1415926D0*DOP)
	RETURN
	END
C-----------------------------------------------------------------------
        SUBROUTINE AMODEL(INMOD,NM1,ND0,TEFF,GLOG
     P  ,DM,TEMP,PRESS,EKONC,ABROSS,AKONC,DENS,HLB)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION DM(NM1),TEMP(NM1),PRESS(NM1),EKONC(NM1),ABROSS(NM1),
     P  AKONC(NM1),DENS(NM1),HLB(NM1)
        CHARACTER*30 INMOD
C
C             MODEL OF THE ATMOSPHERE (NEW KURUCZ)
C
C       DM - MASS DEPTH COORDINATE
C       TEMP - TEMPERATURE
C       PRESS - GASS PRESSURE
C       EKONC - ELECTRON NUMBER DENSITY
C	ABROSS - ROSSELAND MEAN OPACITY [CM^2/G]
C       AKONC - (ALL) ATOM NUMBER DENSITY
C       DENS - DENSITY
C       HLB - GEOMETRICAL DEPTH
C
        OPEN(3,FILE=INMOD,STATUS='OLD')
        READ(3,30) TEFF,GLOG
        READ(3,35) ND0
30      FORMAT(4X,F8.0,9X,F8.5)
35      FORMAT(/////////////////////10X,I3)
        BOL=1.38062D-16
C               WMM=SUM_el (M_el*ABUNDANCE_el)  IS THE MEAN ATOM MASS
C       WHERE ABUNDANCE_el IS N_el/AKONC
C       FOR 90% H + 10 % HE  :   
        WMM=2.2034D-24
        HLB(1)=0.D0
        DO 40 I=1,ND0
          READ(3,*)DM(I),TEMP(I),PRESS(I),EKONC(I),ABROSS(I)
          AKONC(I)=PRESS(I)/TEMP(I)/BOL
          AKONC(I)=AKONC(I)-EKONC(I)
          DENS(I)=AKONC(I)*WMM
          IF(I.GT.1) THEN
            DELTA=2.D0*(DM(I)-DM(I-1))/(DENS(I)+DENS(I-1))
            HLB(I)=DELTA+HLB(I-1)
          ENDIF
40      CONTINUE
        CLOSE(3)
        RETURN
        END
C-----------------------------------------------------------------------
        SUBROUTINE HMODEL(INMOD,NM1,NDPTH
     P  ,DEPTH,TEMP,ELEC,AKONC,DENS)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION DEPTH(NM1),TEMP(NM1),ELEC(NM1),
     P  AKONC(NM1),DENS(NM1),X(500)
        CHARACTER*30 INMOD
C
C             MODEL OF THE ATMOSPHERE (HUBENY)
C
C       DEPTH - MASS DEPTH COORDINATE [G/CM^2]
C       TEMP - TEMPERATURE
C       ELEC - ELECTRON NUMBER DENSITY [CGS]
C       AKONC - (ALL) ATOM NUMBER DENSITY [CGS]
C       DENS - DENSITY [CGS]
C
C               WMM=SUM_el (M_el*ABUNDANCE_el)  IS THE MEAN ATOM MASS
C       WHERE ABUNDANCE_el IS N_el/AKONC
C       FOR 90% H + 10 % HE  :
        WMM=2.2034D-24
        OPEN(3,FILE=INMOD,STATUS='OLD')
        READ(3,*) NDPTH,NUMPAR
        READ(3,*) (DEPTH(I),I=1,NDPTH)
        DO 30 ID=1,NDPTH
          READ(3,*) (X(I),I=1,NUMPAR)
          TEMP(ID)=X(1)
          ELEC(ID)=X(2)
          DENS(ID)=X(3)
	  AKONC(ID)=DENS(ID)/WMM
C          WRITE(2,'(5e11.4)')depth(id),temp(id),elec(id),dens(id)
C     P    ,AKONC(ID)
30      CONTINUE
        CLOSE(3)
        RETURN
        END
C-----------------------------------------------------------------------
	SUBROUTINE PNLTE(IAT,IION,ELO,GLO,EUP,GUP,PLO,PUP,PION,NLTE2
     P  ,STAVS,INMOD,INMOD5,NDIM,NBOD,NION1,ZIP)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C	SUBROUTINE TO EXTRACT AND CALCULATE PLO,PUP,PION
C	FROM TLUSTY OUTPUT
C	PLO, PUP, PION -POPULATIONS OF THE LOWER, UPPER LEVEL AND ION
C	WORKS ONLY IF ENERGY LEVELS IN TLUSTY ATOM MODEL ARE ALL
C	IN CM**-1 OR S**-1??
C	IF BOTH LOWER AND UPPER LEVEL NLTE POPULATION ARE NOT
C	AVAILABLE EXIT SUBROUTINE AND GO ON AS IN PURE LTE (NLTE2=0).
C	IF NLTE POPULATION OF THE LOWER OR UPPER LEVEL IS AVAILABLE
C	(NLTE2=1) THEN CALCULATE PION LIKE A SUM OF ALL LEVELS AND
C	EXTRACT PLO AND PUP. IF ONE OF THE TWO IS NOT AVAILABLE
C	CALCULATE IT:
C       - IN LTE RELATIVELY TO PION (WHICH IS IN NLTE)?? 
C	  (NLTE3=0) OR
C	- IN LTE RELATIVELY TO GROUND STATE OF THE NEXT HIGHER ION
C	  IF IT WAS TREATED AS EXPLICIT LEVEL (I.E. INCLUDED IN 
C	  THE MODEL ATOM WITH ITS OPACITY...) (NLTE3=1)
C
C	IION=1,2,... - NEUTRAL,... 
C	ELO,EUP -ARE IN [1/CM]
C	ZIP -ARE IN [EV]
C	ILO, IUP -ORDER OF THE LOWER, UPPER LEVEL IN TLUSTY MODEL
C
	DIMENSION ZIP(NION1-1),STAVS(NDIM,NION1),X(500),POM(500)
     P  ,PLO(NDIM),PUP(NDIM),PION(NDIM)
	CHARACTER*30 INMOD,INMOD5,FILEI
	CHARACTER*4 A4
	CLIGHT=2.997925D10
	EVCM=8.06554477D3
	NLTE2=0
	NLTE3=0
	ILO=0
	IUP=0
C	READS FROM ?.5 MODEL INFO
        OPEN(7,FILE=INMOD5,STATUS='OLD')
10	READ(7,'(A4)',END=30)A4
	IF(A4.EQ.'*iat')GOTO 20
	GOTO 10
20      READ(7,'(A4)')A4
	IF(A4.NE.'*   ')GOTO 30
C		FINDING ILO AND IUP:
C	FILEI - TLUSTY INPUT FILE WITH THE ATOM MODEL
C	NLEVSI - NO. OF EXPLICIT LEVELS CONSIDERED FOR THE ION
	NLEVS=0
	NLEVSI=0
	ILEVS=0
40	READ(7,*,ERR=30,END=60)IATI,IZI,ILEV,ILASTI,
     P  ILVLIN,NONSTD,A4,FILEI
	IF(IATI.EQ.0)GOTO 60
	IF(IATI.EQ.IAT.AND.IZI.EQ.(IION-1).AND.ILASTI.NE.1)THEN
          OPEN(9,FILE=FILEI,STATUS='OLD')
	  READ(9,*)
	  READ(9,*)EXP1,GLEV1
	  IF(EXP1.GT.1.D7)EXP1=EXP1/CLIGHT
	  EXPLO=EXP1-ELO
	  EXPUP=EXP1-EUP
	  DO 50 I=1,ILEV-1
	    READ(9,*)EXP2,GLEV2
   	    IF(EXP2.GT.1.D7)EXP2=EXP2/CLIGHT
	    IF(EXPLO.LE.EXP1.AND.EXPLO.GT.EXP2)THEN
	      IF((EXP1-EXPLO).LT.(EXPLO-EXP2))THEN
	        ILO=I+ILEVS
	        GLOT=GLEV1
	        NLTE2=1
	      ELSE
	        ILO=I+ILEVS+1	  	 
	        GLOT=GLEV2
	        NLTE2=1
 	      ENDIF
 	    ENDIF
	    IF(EXPUP.LE.EXP1.AND.EXPUP.GT.EXP2)THEN
	      IF((EXP1-EXPUP).LT.(EXPUP-EXP2))THEN
	        IUP=I+ILEVS
	        GUPT=GLEV1
	        NLTE2=1
	      ELSE
	        IUP=I+ILEVS+1	  	 
	        GUPT=GLEV2
	        NLTE2=1
 	      ENDIF
 	    ENDIF
	    EXP1=EXP2
	    GLEV1=GLEV2
50 	  CONTINUE
          CLOSE(9)
	  NLEVSI=ILEV
	  NLEVS=ILEVS
	ENDIF
	IF(IATI.EQ.IAT.AND.IZI.EQ.IION.AND.ILASTI.NE.1)THEN
	  NLTE3=1
          OPEN(9,FILE=FILEI,STATUS='OLD')
	  READ(9,*)
	  READ(9,*)EXP3,GLEV3
          CLOSE(9)
	ENDIF
	ILEVS=ILEVS+ILEV
	GOTO 40
C		CALCULATING PION, PLO AND PUP:   
C	OUR ION HAS NLEVSI LEVELS: (NLEVS+1,...,NLEVS+NLEVSI)
C	PGION2 - POPULATION OF THE GROUND STATE OF NEXT - THE HIGHER ION
C	GLEV3 - STAT. WEIGHT OF THE GROUND STATE OF THE NEXT ION
60	CLOSE(7)
	IF(NLTE2.EQ.1)THEN
          OPEN(3,FILE=INMOD,STATUS='OLD')
          READ(3,*) NDPTH,NUMPAR
          READ(3,*) (POM(I),I=1,NDPTH)
	  POT=ZIP(IION)*EVCM
          DO 80 ID=1,NDPTH
            READ(3,*) (X(I),I=1,NUMPAR)
            TEP=X(1)
	    EKONC=X(2)
	    PION(ID)=0.D0
	    DO 70 L=1,NLEVSI
	      PION(ID)=PION(ID)+X(NLEVS+L+3)
70 	    CONTINUE
	    PGION2=X(NLEVS+NLEVSI+4)
	    IF(ILO.GT.0)THEN
              PLO(ID)=X(ILO+3)*GLO/GLOT
	    ELSE
	      IF(NLTE3.EQ.0)THEN
	        PLO(ID)=BOLT(TEP,STAVS(ID,IION),GLO,ELO,1)*PION(ID)
	      ELSE
	        PLO(ID)=SABOF(TEP,EKONC,GLO,GLEV3,ELO,POT,1)*PGION2
	      ENDIF
	    ENDIF
	    IF(IUP.GT.0)THEN
              PUP(ID)=X(IUP+3)*GUP/GUPT
	    ELSE
              IF(NLTE3.EQ.0)THEN
	        PUP(ID)=BOLT(TEP,STAVS(ID,IION),GUP,EUP,1)*PION(ID)
              ELSE
                PUP(ID)=SABOF(TEP,EKONC,GUP,GLEV3,EUP,POT,1)*PGION2
              ENDIF
	    ENDIF
80        CONTINUE
	  CLOSE(3)
	ELSE
	  GOTO 200
	ENDIF
	GOTO 200
30	WRITE(6,*)'ERROR: WRONG FILE: MODEL.HUB.5.'
	STOP
200     RETURN
        END
C-----------------------------------------------------------------------
	DOUBLE PRECISION FUNCTION SABOF(TEMP,EKONC,GI1,GI2,EI1,POT
     P  ,IRIAD)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C             TEMP -TEMPERATURE [KELVIN]                               C
C             EKONC-ELECTRON NUMBER DENSITY [CGS]                      C
C             GI1  -STATISTICAL WEIGHT OF THE LEVEL -l OF ION -i       C
C             GI2  -STAT. WEIGHT OF THE GROUND STATE -g OF ION -i+1    C
C	      EI1  -EXCITATION ENERGY OF THE LEVEL -l [eV] >0          C
C             POT  -IONIZ. POTENTIAL OF ION -i [EV], POT>0             C
C             OUTPUT - POPULATION RATIO N(l,i)/N(g,i+1)                C
C-----------------------OR IF------------------------------------------C
C             IRIAD=1  POT,EI1 ARE IN [1/CM]                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
	EVCM=8.06554477D3
	DIFF=POT-EI1
        IF(IRIAD.EQ.1)THEN
          POM=DEXP(2.302585D0*5039.77D0/TEMP*DIFF/EVCM)
        ELSE
          POM=DEXP(2.302585D0*5039.77D0/TEMP*DIFF)
        ENDIF
	SABOF=2.0706E-16/TEMP/DSQRT(TEMP)*GI1/GI2*POM*EKONC
        RETURN
        END
C-----------------------------------------------------------------------
	SUBROUTINE PFIRW(NDIM,NBOD,NION1,NION,STA,TEMP,STAVS)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C	PARAMETER(NION1=9)
        DIMENSION STA(6,NION1),STAVS(NDIM,NION1),TEMP(NDIM)
        DO 80 I=1,NBOD
	  DO 70 J=1,NION
	    IF (ABS(STA(1,J)-0.D0).LT.1.D-30) GO TO 60
	    SALA=DLOG(TEMP(I))
	    ARG=STA(1,J)+STA(2,J)*SALA+STA(3,J)*SALA*SALA+
     P      STA(4,J)*SALA**3.D0+STA(5,J)*SALA**4.D0+STA(6,J)*SALA**5.D0
	    STAVS(I,J)=DEXP(ARG)
	    GO TO 70
60	    STAVS(I,J)=1.D0
70	  CONTINUE
80      CONTINUE
        WRITE (2,'(A,A)')' TEM     STAVS1    STAVS2    STAVS3    STAVS4'
     P,'    STAVS5    STAVS6    STAVS7    STAVS8    STAVS9'
	DO 140 I=1,NBOD
	  WRITE(2,130)TEMP(I),(STAVS(I,J),J=1,NION)
130       FORMAT(F7.0,20E10.3) 
140     CONTINUE
	RETURN
	END
C-----------------------------------------------------------------------
        subroutine potlow(TEMP,ane,eplow)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c       TEMP - Temperature[K]
c       ane - electron number density [cm^{-3}]
c       lowering of the ionization potential for neutrals, z=1 in [eV]
c       for ions(z):      eplow(z)=z*eplow(1)
        BOL=1.38062D-16                      
        PI=3.14159D0
        e=4.80325d-10
        pom=bol*TEMP/8.d0/pi/ane
c       debye lenght in [CGS]
        debye=sqrt(pom)/e
        eplow=1.44d-7/debye
        RETURN
        END
C-----------------------------------------------------------------------
	SUBROUTINE PFDWOR(NDIM,NBOD,NION1,NION,IAT,TEMP,EKONC,STAVS)
c
c       partition functions after:
c       M.M.Dworetsky & B. Smalley, private communication
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	DIMENSION STAVS(NDIM,NION1),EKONC(NDIM),TEMP(NDIM),U(5)
        DO 80 I=1,NBOD
          CALL POTLOW(TEMP(I),EKONC(I),EPLOW)
          THETA=5039.77D0/TEMP(I)
C		PARTFN IS IN 'pfdwor.inc'
          CALL PARTFN(IAT,THETA,U,EPLOW)
          DO 70 J=1,NION
	    IF(J.LE.5)THEN
	      STAVS(I,J)=U(J)
	    ELSE
	      STAVS(I,J)=1.D0
	    ENDIF
70        CONTINUE
80      CONTINUE
        WRITE (2,'(A,A)')' TEM     STAVS1    STAVS2    STAVS3    STAVS4'
     P,'    STAVS5    STAVS6    STAVS7    STAVS8    STAVS9'
        DO 140 I=1,NBOD
          WRITE(2,130)TEMP(I),(STAVS(I,J),J=1,NION)
130       FORMAT(F7.0,20E10.3)
140     CONTINUE
        RETURN
        END
C-----------------------------------------------------------------------
	SUBROUTINE ZASTUP(DM,TEMP,EKONC,ZIP,STAVS,RR,RION,R1,NDIM,NBOD
     P  ,NION1,NION)
C       RR(I,J)- POPULATION OF THE J-TH ION/TOTAL ELEMENT POLULATION
C		 AT I-TH DEPTH
C	R1(J)= N(J)/N(1)
C	RION(J)= N(J)/N(J-1)
C	SAH	-CALLING FUNCTION
C	
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	DIMENSION ZIP(NION1-1),STAVS(NDIM,NION1)
     P  ,DM(NDIM),TEMP(NDIM),EKONC(NDIM)
     P  ,RION(NION1),R1(NION1),RR(NDIM,NION1)
	DO 10 J=1,NION1
	  DO 5 I=1,NBOD
	    RR(I,J)=1.D-31
5	  CONTINUE
10	CONTINUE
        DO 40 I=1,NBOD
	  RION(1)=1
	  R1(1)=1
	  SUM1=1.D0
	  DO 20 J=2,NION
	    RION(J)=SAH(TEMP(I),EKONC(I),ZIP(J-1),STAVS(I,J-1)
     P      ,STAVS(I,J))
            R1(J)=R1(J-1)*RION(J)
	    SUM1=SUM1+R1(J)
20	  CONTINUE
	  DO 30 J=1,NION
	    RR(I,J)=R1(J)/SUM1
30	  CONTINUE
40	CONTINUE
         WRITE(2,'(A,40I11)')' DM',(I,I=1,NION)
	DO 50 I=1,NBOD
	  WRITE(2,130)DM(I),(RR(I,J),J=1,NION)
130	  FORMAT(40(D10.3,1X))
50	CONTINUE
	RETURN
	END
C-----------------------------------------------------------------------
        DOUBLE PRECISION FUNCTION SAH(TE,EK,ZP,S1,S2)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C             SAHA EQUATION (RATIO OF THE TWO NEIGHBOUR ION NUMBER
C       DENSITIES)
C             ZP -IV [EV],BOLT.CONST.-IN [ERG/K]
        BOL=1.3806503D-16
        SAH=1.2020D9*S2/(S1*(5039.77D0/TE)**2.5D0
     P  *DEXP(2.302585D0*5039.77D0/TE*ZP)*EK*TE*BOL)
        RETURN   
        END      
C-----------------------------------------------------------------------
	INCLUDE 'pfdwor.inc'
